@@ -6,14 +6,16 @@ from jax_fdm import DTYPE_JAX
 from jax_fdm.parameters import split
 from jax_fdm.parameters import combine
 
+from jax_fdm.parameters import ParameterGroup
+
 from jax_fdm.parameters import EdgeParameter
-from jax_fdm.parameters import NodeAnchorParameter
+from jax_fdm.parameters import NodeSupportParameter
 from jax_fdm.parameters import NodeLoadParameter
 
 from jax_fdm.parameters import EdgeForceDensityParameter
-from jax_fdm.parameters import NodeAnchorXParameter
-from jax_fdm.parameters import NodeAnchorYParameter
-from jax_fdm.parameters import NodeAnchorZParameter
+from jax_fdm.parameters import NodeSupportXParameter
+from jax_fdm.parameters import NodeSupportYParameter
+from jax_fdm.parameters import NodeSupportZParameter
 from jax_fdm.parameters import NodeLoadXParameter
 from jax_fdm.parameters import NodeLoadYParameter
 from jax_fdm.parameters import NodeLoadZParameter
@@ -31,9 +33,9 @@ class ParameterManager:
         A list of optimization parameters.
     """
     parameter_types = [EdgeForceDensityParameter,
-                       NodeAnchorXParameter,
-                       NodeAnchorYParameter,
-                       NodeAnchorZParameter,
+                       NodeSupportXParameter,
+                       NodeSupportYParameter,
+                       NodeSupportZParameter,
                        NodeLoadXParameter,
                        NodeLoadYParameter,
                        NodeLoadZParameter]
@@ -50,12 +52,14 @@ class ParameterManager:
         self._indices_opt = None
         self._indices_frozen = None
         self._indices_optfrozen = None
+        self._indices_groups = None
 
         self._indices_fdm = None
         self._indices_fd = None
         self._indices_xyzfixed = None
         self._indices_loads = None
 
+        self._parameters_value = None
         self._parameters_ordered = None
         self._parameters_model = None
         self._parameters_opt = None
@@ -77,12 +81,17 @@ class ParameterManager:
     def init(self):
         """
         Initialiaze the properties of this object so that it is static after this call.
+
+        TODO: This is fairly anti-pythonic. Please refactor me.
         """
         self.indices_fdm
         self.indices_optfrozen
+        self.parameters
         self.parameters_model
         self.parameters_opt
         self.parameters_frozen
+        self.parameters_ordered
+        self.indices_groups
 
 # ==========================================================================
 # Starting indices
@@ -152,6 +161,24 @@ class ParameterManager:
             self._indices_loads = np.array(range(start, stop))
         return self._indices_loads
 
+    @property
+    def indices_groups(self):
+        """
+        A list with indices distributions optimization parameters to parameter groups.
+        """
+        if self._indices_groups is None:
+            indices = []
+            for idx, parameter in enumerate(self.parameters_ordered):
+                if isinstance(parameter, ParameterGroup):
+                    for j in range(len(parameter.key)):
+                        indices.append(idx)
+                else:
+                    indices.append(idx)
+
+            self._indices_groups = np.array(indices, dtype=np.int64)
+
+        return self._indices_groups
+
     @property
     def indices_opt(self):
         """
@@ -160,9 +187,15 @@ class ParameterManager:
         if self._indices_opt is None:
             indices = []
             pshift = 0
+            # iterate over ordered parameter classes typed
             for ptype in self.parameter_types:
+                # get the indices of the all the parameters of this parameter class
                 itype = self._indices_type(ptype)
-                indices.extend(self._indices_shift(itype, pshift))
+                # shift class indices by class shift
+                itype_shifted = self._indices_shift(itype, pshift)
+                # store shifted class indices in indices list
+                indices.extend(itype_shifted)
+                # update class shift
                 pshift += self._shift_type(ptype)
 
             self._indices_opt = indices
@@ -193,9 +226,15 @@ class ParameterManager:
         indices = []
         for parameter in self.parameters:
             if isinstance(parameter, cls):
-                indices.append(parameter.index(self.model))
+                if isinstance(parameter, ParameterGroup):
+                    indices.extend(parameter.index(self.model))
+                else:
+                    indices.append(parameter.index(self.model))
 
-        return np.array(indices)
+        # assert len(indices) > 0, "Could not compute parameter indices per type. Are these valid parameters?"
+
+        # return np.array(indices, dtype=np.int64)
+        return indices
 
     def _indices_shift(self, indices, shift):
         """
@@ -209,7 +248,7 @@ class ParameterManager:
         """
         if issubclass(ptype, EdgeParameter):
             shift = self.network.number_of_edges()
-        elif issubclass(ptype, NodeAnchorParameter):
+        elif issubclass(ptype, NodeSupportParameter):
             shift = self.network.number_of_anchors()
         elif issubclass(ptype, NodeLoadParameter):
             shift = self.network.number_of_nodes()
@@ -248,6 +287,19 @@ class ParameterManager:
 
         return self._bounds_up
 
+    @property
+    def parameters_value(self):
+        """
+        Return an array with the intial value of the optimization parameters.
+        """
+        if self._parameters_value is None:
+            values = []
+            for parameter in self.parameters_ordered:
+                values.append(parameter.value(self.model))
+            self._parameters_value = jnp.array(values, dtype=DTYPE_JAX)
+
+        return self._parameters_value
+
 # ==========================================================================
 # Parameters
 # ==========================================================================
@@ -258,11 +310,16 @@ class ParameterManager:
         The optimization parameter objects, sorted by type.
         """
         if self._parameters_ordered is None:
+
             parameters = []
             for ptype in self.parameter_types:
+
                 for parameter in self.parameters:
                     if isinstance(parameter, ptype):
                         parameters.append(parameter)
+
+            assert len(parameters) > 0, "No valid optimization parameters defined!"
+
             self._parameters_ordered = parameters
 
         return self._parameters_ordered
@@ -270,7 +327,7 @@ class ParameterManager:
     @property
     def parameters_model(self):
         """
-        The model parameters.
+        The model parameters as a single array.
         """
         if self._parameters_model is None:
             param_arrays = []
@@ -307,8 +364,9 @@ class ParameterManager:
 
     def parameters_fdm(self, params_opt):
         """
-        Reshape optimizable model parameters into fdm parameters.
+        Convert optimization parameters into fdm parameters.
         """
+        params_opt = params_opt[self.indices_groups]
         params = combine(params_opt, self.parameters_frozen, adef=self.indices_optfrozen)
         q, xyz_fixed, loads = jnp.split(params, self.indices_fdm)
 
